#include <iostream>
#include <cstring>

#include "LecteurNFC.h"
#include "CarteMifare.h"

using namespace std;

//#define BADGE

#ifdef BADGE
static uint8_t mesCles[][6] =
{
    {0xff, 0xff, 0xff, 0xff, 0xff, 0xff},
    {0xff, 0xff, 0xff, 0xff, 0xff, 0xff},
    {0xff, 0xff, 0xff, 0xff, 0xff, 0xff},
    {0xff, 0xff, 0xff, 0xff, 0xff, 0xff},
    {0xff, 0xff, 0xff, 0xff, 0xff, 0xff},
    {0x6a, 0x19, 0x87, 0xc4, 0x0a, 0x21},
    {0x6a, 0x19, 0x87, 0xc4, 0x0a, 0x21},
    {0x6a, 0x19, 0x87, 0xc4, 0x0a, 0x21},
    {0x6a, 0x19, 0x87, 0xc4, 0x0a, 0x21},
    {0x6a, 0x19, 0x87, 0xc4, 0x0a, 0x21},
    {0x6a, 0x19, 0x87, 0xc4, 0x0a, 0x21},
    {0x6a, 0x19, 0x87, 0xc4, 0x0a, 0x21},
    {0x6a, 0x19, 0x87, 0xc4, 0x0a, 0x21},
    {0x6a, 0x19, 0x87, 0xc4, 0x0a, 0x21},
    {0x6a, 0x19, 0x87, 0xc4, 0x0a, 0x21},
    {0x6a, 0x19, 0x87, 0xc4, 0x0a, 0x21}
};

static uint8_t cle[]                = {0x6a, 0x19, 0x87, 0xc4, 0x0a, 0x21};
#else
static uint8_t cle[]                = {0x00, 0x00, 0x00, 0x00, 0x00, 0x01};
//static uint8_t droitsAcces[]        = {0x78, 0x77, 0x88, 0x69};
static uint8_t droitsAcces[]        = {0x7F, 0x07, 0x88, 0x69};
static uint8_t donneesBloc[]        = {'B', 'o', 'n', 'j', 'o', 'u', 'r', ' ', 'l', 'e', ' ', 'm', 'o', 'n', 'd', 'e'};
static uint8_t donneesBlocVierge[]  = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
#endif

//#define RESET

#ifdef RESET
static uint8_t uid[] = {0x01,  0x23,  0x45,  0x67}; // 01 23 45 67
//static uint8_t uid[] = {0x00,  0x00,  0x00,  0x00}; // 00 00 00 00
#endif

int main(int argc, char *argv[])
{
    /* Exemples d'utilisation des classes LecteurNFC et CarteMifare */
    CarteMifare *carteMifare = new CarteMifare; // utilise les clés par défaut
    LecteurNFC lecteurNFC(carteMifare);
   
#ifdef RESET
    lecteurNFC.reset(uid); // ou : lecteurNFC.reset(uid, true);

    delete carteMifare;
    return 0;
#endif

    cout << "-> Test lecteur NFC" << endl;
    cout << "Version libnfc : " << lecteurNFC.getLibVersion() << endl;   

    //cout << "Lecteur : " << lecteurNFC.getNomLecteur() << endl;
    //cout << "Peripherique : " << lecteurNFC.getNomPeripherique() << endl;
    // ou :
    lecteurNFC.afficherDetails();

    //lecteurNFC.afficherInfosCarte();
    lecteurNFC.afficherInfosCarte(true); // affichage détaillé

#ifdef BADGE
    // fixe les clés pour ce badge
    for(int i = 0; i < carteMifare->getNbBlocs(); i++)
    {
        carteMifare->setCle('A', mesCles[i/4], i);
    }
#endif

    cout << endl << "-> Test lecture CarteMifare" << endl;
    // un numéro de bloc au hasard !
    int x = 60;

    // lit la carte présente sur le lecteur
    if(lecteurNFC.lireCarteMifare())
    {
        // affiche le contenu de la carte
        carteMifare->afficherBlocs();

        cout << "-> Test lecture bloc 0" << endl;
        // affiche le contenu du bloc 0 de la carte
        carteMifare->afficherBlocFabricant();
        
        cout << "-> Test lecture bloc sécurité " << x << endl;
        // affiche le contenu du bloc sécurité associé au bloc x
        carteMifare->afficherBlocSecurite(x);
    }

    // on efface ce qui a été lu précdemment
    carteMifare->effacerBlocsLus();
    // lit le bloc x de la carte présente sur le lecteur
    if(lecteurNFC.lireBlocCarteMifare(x))
    {
        cout << "-> Test lecture bloc " << x << endl;
        // affiche le contenu du bloc x
        carteMifare->afficherBloc(x);
    }

#ifdef BADGE

    // on efface le bloc x lu précdemment
    carteMifare->effacerBlocLu(x);
    // lit le bloc x avec la clé c de la carte présente sur le lecteur
    if(lecteurNFC.lireBlocCarteMifare(x, cle))
    {
        // affiche le contenu du bloc x
        carteMifare->afficherBloc(x);
    }

#endif

#ifndef BADGE

    cout << endl << "-> Test écriture CarteMifare" << endl;
    MifareBloc bloc; // le bloc à écrire sur la carte
    int n = 60; // un numéro de bloc où l'on va écrire

    // on recopie des données vierges dans le bloc
    memcpy(bloc.mbData.data, donneesBlocVierge, 16); // que des 0
    // écrit le bloc n sur la carte présente sur le lecteur
    if(lecteurNFC.ecrireBlocCarteMifare(n, bloc))
    {
        // affiche après le contenu du bloc n
        carteMifare->afficherBloc(n);
    }

    // on recopie des données dans le bloc
    memcpy(bloc.mbData.data, donneesBloc, 16); // "Bonjour le monde"
    // écrit le bloc n sur la carte présente sur le lecteur
    if(lecteurNFC.ecrireBlocCarteMifare(n, bloc))
    {
        // affiche après le contenu du bloc n
        carteMifare->afficherBloc(n);
    }

#endif

#ifdef ECRITURE_CLE

    cout << endl << "Test écriture clé CarteMifare" << endl;
    int c = 7; // un numéro de bloc où l'on va écrire la nouvelle clé

    // sauf la première fois !
    carteMifare->setCle('B', cle, c);
    
    // écrit la nouvelle clé B dans le boc c
    if(lecteurNFC.ecrireCleCarteMifare(c, cle, 'B'))
    {
        // affiche le contenu du bloc sécurité c
        carteMifare->afficherBlocSecurite(c);
    }

    // écrit les nouveaux droits d'acccès dans le boc c
    if(lecteurNFC.ecrireDroitsAccesCarteMifare(c, droitsAcces, 'B'))
    {
        // affiche le contenu du bloc sécurité c
        carteMifare->afficherBlocSecurite(c);
    }
    
#endif

    delete carteMifare;

    return 0;
}


