#include "CarteMifare.h"

#include <QDebug>

CarteMifare::CarteMifare() : nbBlocs(NB_BLOCS), nbCles(NB_CLES)
{
    uint8_t default_acl[] = {0xff, 0x07, 0x80, 0x69};

    memcpy(parametres.mpAuthentication.key, defaultKeys[CLE_DEFAUT], 6);
    memcpy(parametres.mpAuthentication.authUid, default_acl, 4);

    nm.nmt = NMT_ISO14443A;
    nm.nbr = NBR_106;

    typeAccesCle = 'A';    

    memset(blocsLus, 0, sizeof(blocsLus));
    nbBlocsLus = 0;
    this->uid = 0;

    for(int c = 0; c < nbCles; c++)
    {
        memcpy(clesA[c], defaultKeys[CLE_DEFAUT], 6);
        memcpy(clesB[c], defaultKeys[CLE_DEFAUT], 6);
    }
}

CarteMifare::CarteMifare(MifareNumCle numCle) : nbBlocs(NB_BLOCS), nbCles(NB_CLES)
{
    uint8_t default_acl[] = {0xff, 0x07, 0x80, 0x69};

    if(numCle >= CLE_DEFAUT && numCle < NB_MIFARE_CLES)
        memcpy(parametres.mpAuthentication.key, defaultKeys[numCle], 6);
    else
        memcpy(parametres.mpAuthentication.key, defaultKeys[CLE_DEFAUT], 6);
    memcpy(parametres.mpAuthentication.authUid, default_acl, 4);

    nm.nmt = NMT_ISO14443A;
    nm.nbr = NBR_106;

    typeAccesCle = 'A';

    memset(blocsLus, 0, sizeof(blocsLus));
    nbBlocsLus = 0;
    this->uid = 0;

    for(int c = 0; c < nbCles; c++)
    {
        if(numCle >= CLE_DEFAUT && numCle < NB_MIFARE_CLES)
        {
            memcpy(clesA[c], defaultKeys[numCle], 6);
            memcpy(clesB[c], defaultKeys[numCle], 6);
        }
    }
}

CarteMifare::CarteMifare(char typeAccesCle, MifareNumCle numCle) : nbBlocs(NB_BLOCS), nbCles(NB_CLES)
{
    uint8_t default_acl[] = {0xff, 0x07, 0x80, 0x69};

    if(numCle >= CLE_DEFAUT && numCle < NB_MIFARE_CLES)
        memcpy(parametres.mpAuthentication.key, defaultKeys[numCle], 6);
    else
        memcpy(parametres.mpAuthentication.key, defaultKeys[CLE_DEFAUT], 6);
    memcpy(parametres.mpAuthentication.authUid, default_acl, 4);

    nm.nmt = NMT_ISO14443A;
    nm.nbr = NBR_106;

    this->typeAccesCle = typeAccesCle;

    memset(blocsLus, 0, sizeof(blocsLus));
    nbBlocsLus = 0;
    this->uid = 0;

    for(int c = 0; c < nbCles; c++)
    {
        if(numCle >= CLE_DEFAUT && numCle < NB_MIFARE_CLES)
        {
            memcpy(clesA[c], defaultKeys[numCle], 6);
            memcpy(clesB[c], defaultKeys[numCle], 6);
        }
    }
}

CarteMifare::~CarteMifare()
{
}

void CarteMifare::effacerBlocsLus()
{
    memset(blocsLus, 0, sizeof(blocsLus));
    nbBlocsLus = 0;
}

void CarteMifare::effacerBlocLu(int numeroBloc)
{
    if(numeroBloc >= 0 && numeroBloc < NB_BLOCS && blocsLus[numeroBloc] == true)
    {
        blocsLus[numeroBloc] = false;
        nbBlocsLus--;
    }
}

MifareParametres CarteMifare::getParametres() const
{
    return parametres;
}

void CarteMifare::setParametres(MifareParametres parametres)
{
    memcpy(&this->parametres, &parametres, sizeof(parametres));
}


void CarteMifare::afficherParametresAuthentication() const
{
    qDebug() << CarteMifare::toQStringHexa(parametres.mpAuthentication.key, 6, "* Key : ", " ");
    //CarteMifare::printHexa(parametres.mpAuthentication.key, 6, "* Key : ");
    qDebug() << CarteMifare::toQStringHexa(parametres.mpAuthentication.authUid, 4, "* UID : ", " ");
    //CarteMifare::printHexa(parametres.mpAuthentication.authUid, 4, "* UID : ");
}

void CarteMifare::setTypeAccesCle(char typeAccesCle)
{
    if(typeAccesCle == 'A' || typeAccesCle == 'B')
        this->typeAccesCle = typeAccesCle;
}

MifareCle CarteMifare::getCle(char typeAccesCle/*='A'*/, int numeroBloc/*=0*/) const
{
    MifareCle cle;

    int n = getNumeroBlocSecurite(numeroBloc);
    if(typeAccesCle == 'A')
    {
        memcpy(cle.key, clesA[n/4], 6);
    }
    else if(typeAccesCle == 'B')
    {
        memcpy(cle.key, clesB[n/4], 6);
    }

    return cle;
}

void CarteMifare::setCle(MifareNumCle numCle, int numeroBloc/*=0*/)
{
    if(numCle >= CLE_DEFAUT && numCle < NB_MIFARE_CLES)
    {
        memcpy(parametres.mpAuthentication.key, defaultKeys[numCle], 6);
        int n = getNumeroBlocSecurite(numeroBloc);
        memcpy(clesA[n/4], parametres.mpAuthentication.key, 6);
        memcpy(clesB[n/4], parametres.mpAuthentication.key, 6);
    }
}

void CarteMifare::setCle(uint8_t cle[], int numeroBloc/*=0*/)
{
    memcpy(parametres.mpAuthentication.key, cle, 6);
    int n = getNumeroBlocSecurite(numeroBloc);
    memcpy(clesA[n/4], parametres.mpAuthentication.key, 6);
    memcpy(clesB[n/4], parametres.mpAuthentication.key, 6);
}

void CarteMifare::setCle(char typeAccesCle, MifareNumCle numCle, int numeroBloc/*=0*/)
{
    setTypeAccesCle(typeAccesCle);
    if(numCle >= CLE_DEFAUT && numCle < NB_MIFARE_CLES)
    {
        memcpy(parametres.mpAuthentication.key, defaultKeys[numCle], 6);
        int n = getNumeroBlocSecurite(numeroBloc);
        if(typeAccesCle == 'A')
        {
            memcpy(clesA[n/4], parametres.mpAuthentication.key, 6);
        }
        else if(typeAccesCle == 'B')
        {
            memcpy(clesB[n/4], parametres.mpAuthentication.key, 6);
        }
    }
}

void CarteMifare::setCle(char typeAccesCle, uint8_t cle[], int numeroBloc/*=0*/)
{
    setTypeAccesCle(typeAccesCle);
    memcpy(parametres.mpAuthentication.key, cle, 6);
    int n = getNumeroBlocSecurite(numeroBloc);
    if(typeAccesCle == 'A')
    {
        memcpy(clesA[n/4], parametres.mpAuthentication.key, 6);
    }
    else if(typeAccesCle == 'B')
    {
        memcpy(clesB[n/4], parametres.mpAuthentication.key, 6);
    }
}


MifareBloc CarteMifare::getBloc(int numeroBloc) const
{
    if(numeroBloc >= 0 && numeroBloc < NB_BLOCS && blocsLus[numeroBloc] == true)
        return blocs[numeroBloc];

    MifareBloc b;
    return b;
}

MifareBloc CarteMifare::getBlocSecurite(int numeroBloc) const
{
    int n = getNumeroBlocSecurite(numeroBloc);
    if(n >= 0 && n < NB_BLOCS && blocsLus[n] == true)
        return blocs[n];

    MifareBloc b;
    return b;
}

MifareBloc CarteMifare::getBlocFabricant() const
{
    if(blocsLus[0] == true)
        return blocs[0];

    MifareBloc b;
    return b;
}

void CarteMifare::setBloc(int numeroBloc, MifareBloc bloc)
{
    if(numeroBloc >= 0 && numeroBloc < NB_BLOCS)
    {
        memcpy(&blocs[numeroBloc], &bloc, sizeof(MifareBloc));
        if(blocsLus[numeroBloc] == false)
        {
            blocsLus[numeroBloc] = true;
            nbBlocsLus++;
        }
    }
}

void CarteMifare::afficherBlocs() const
{
    int i;

    if(nbBlocsLus == 0)
        return;

    qDebug() << endl << "Contenu de la carte :";
    for(i = 0; i < NB_BLOCS; i++)
    {
        afficherBloc(i);
    }
    qDebug() <<  endl;
}

QString CarteMifare::getBlocToString(int numeroBloc) const
{
    QString bloc = "";
    if(numeroBloc >= 0 && numeroBloc < NB_BLOCS && blocsLus[numeroBloc] == true)
    {
        for(int i=0;i<16;i++)
        {
            if(isprint(blocs[numeroBloc].mbData.data[i]))
            {
                bloc += QChar(blocs[numeroBloc].mbData.data[i]);
            }
            else
            {
                bloc += QChar('.');
            }
        }
    }
    return bloc;
}

QString CarteMifare::getBlocToHexaString(int numeroBloc) const
{
    QString bloc = "";
    if(numeroBloc >= 0 && numeroBloc < NB_BLOCS && blocsLus[numeroBloc] == true)
    {
        bloc += CarteMifare::toQStringHexa(blocs[numeroBloc].mbData.data, 16, QString("%1").arg(numeroBloc, 2, 10, QLatin1Char('0')) + " : ", " ");

        if(numeroBloc == 0)
        {

        }
        else if(estPremierBloc(numeroBloc))
        {

        }
        else if(estBlocSecurite(numeroBloc))
        {

        }
        else
        {

        }
    }
    return bloc;
}

QString CarteMifare::afficherBloc(int numeroBloc) const
{
    QString bloc = "";
    if(numeroBloc >= 0 && numeroBloc < NB_BLOCS && blocsLus[numeroBloc] == true)
    {
        int n = getNumeroBlocSecurite(numeroBloc);
        QString details = "";
        details += "type " + QString(typeAccesCle) + ", ";
        if(typeAccesCle == 'A')
        {
            details += "cle " + CarteMifare::toQStringHexa(clesA[n/4], 6);
        }
        else if(typeAccesCle == 'B')
        {
            details += "cle " + CarteMifare::toQStringHexa(clesB[n/4], 6);
        }
        details += " : ";

        if(numeroBloc == 0)
        {
            qDebug() << "* [f] Secteur " + QString("%1").arg((numeroBloc/4), 2, 10, QLatin1Char('0')) + " " + CarteMifare::toQStringHexa(blocs[numeroBloc].mbData.data, 16, ", bloc " + QString("%1").arg(numeroBloc, 2, 10, QLatin1Char('0')) + ", " + details, " ");
            bloc += "* [f] Secteur " + QString("%1").arg((numeroBloc/4), 2, 10, QLatin1Char('0')) + " " + CarteMifare::toQStringHexa(blocs[numeroBloc].mbData.data, 16, ", bloc " + QString("%1").arg(numeroBloc, 2, 10, QLatin1Char('0')) + ", " + details, " ");
        }
        else if(estPremierBloc(numeroBloc))
        {
            qDebug() << "* [d] Secteur " + QString("%1").arg((numeroBloc/4), 2, 10, QLatin1Char('0')) + " " + CarteMifare::toQStringHexa(blocs[numeroBloc].mbData.data, 16, ", bloc " + QString("%1").arg(numeroBloc, 2, 10, QLatin1Char('0')) + ", " + details, " ");
            bloc += "* [d] Secteur " + QString("%1").arg((numeroBloc/4), 2, 10, QLatin1Char('0')) + " " + CarteMifare::toQStringHexa(blocs[numeroBloc].mbData.data, 16, ", bloc " + QString("%1").arg(numeroBloc, 2, 10, QLatin1Char('0')) + ", " + details, " ");
        }
        else if(estBlocSecurite(numeroBloc))
        {
            qDebug() << "* [s] Secteur " + QString("%1").arg((numeroBloc/4), 2, 10, QLatin1Char('0')) + " " + CarteMifare::toQStringHexa(blocs[numeroBloc].mbData.data, 16, ", bloc " + QString("%1").arg(numeroBloc, 2, 10, QLatin1Char('0')) + ", " + details, " ");
            bloc += "* [s] Secteur " + QString("%1").arg((numeroBloc/4), 2, 10, QLatin1Char('0')) + " " + CarteMifare::toQStringHexa(blocs[numeroBloc].mbData.data, 16, ", bloc " + QString("%1").arg(numeroBloc, 2, 10, QLatin1Char('0')) + ", " + details, " ");
        }
        else
        {
            qDebug() << "* [d] Secteur " + QString("%1").arg((numeroBloc/4), 2, 10, QLatin1Char('0')) + " " + CarteMifare::toQStringHexa(blocs[numeroBloc].mbData.data, 16, ", bloc " + QString("%1").arg(numeroBloc, 2, 10, QLatin1Char('0')) + ", " + details, " ");
            bloc += "* [d] Secteur " + QString("%1").arg((numeroBloc/4), 2, 10, QLatin1Char('0')) + " " + CarteMifare::toQStringHexa(blocs[numeroBloc].mbData.data, 16, ", bloc " + QString("%1").arg(numeroBloc, 2, 10, QLatin1Char('0')) + ", " + details, " ");
        }
    }
    return bloc;
}

void CarteMifare::afficherBlocSecurite(int numeroBloc) const
{
    int n = getNumeroBlocSecurite(numeroBloc);

    afficherBloc(n);

    if(n >= 0 && n < NB_BLOCS && blocsLus[n] == true)
    {
        qDebug() <<  "Details : ";
        MifareBloc b = blocs[n];
        qDebug() << CarteMifare::toQStringHexa(b.mbSecurite.keyA, 6, "* [s] Bloc " + QString::number(n) + " - Key A : ", " ");
        qDebug() << CarteMifare::toQStringHexa(b.mbSecurite.accessBits, 4, "* [s] Bloc " + QString::number(n) + " - Access bits : ", " ");        
        qDebug() << CarteMifare::toQStringHexa(b.mbSecurite.keyB, 6, "* [s] Bloc " + QString::number(n) + " - Key B : ", " ");

        decoderACL(b.mbSecurite.accessBits);
        qDebug() <<  endl;
    }
}

void CarteMifare::afficherBlocFabricant() const
{
    afficherBloc(0);

    if(blocsLus[0] == true)
    {
        qDebug() <<  "Details : ";
        MifareBloc b = blocs[0];
        qDebug() << CarteMifare::toQStringHexa(b.mbFabricant.uid, 4, "* [f] Bloc " + QString::number(0) + " - UID : ", " ") << "(" << CarteMifare::toQStringNum(b.mbFabricant.uid, 4) << ")";
        qDebug() << CarteMifare::toQStringHexa(&b.mbFabricant.bcc, 1, "* [f] Bloc " + QString::number(0) + " - BCC : 0x") << "(" << CarteMifare::toQStringNum(&b.mbFabricant.bcc, 1) << ")";
        qDebug() << CarteMifare::toQStringHexa(b.mbFabricant.atqa, 2, "* [f] Bloc " + QString::number(0) + " - ATQA : ", " ");
        uint8_t calcul_bcc = b.mbData.data[0] ^ b.mbData.data[1] ^ b.mbData.data[2] ^ b.mbData.data[3];
        qDebug() << CarteMifare::toQStringHexa(&calcul_bcc, 1, "* [f] Bloc " + QString::number(0) + " - Verification BCC : 0x") << "(" << CarteMifare::toQStringNum(&calcul_bcc, 1) << ")";
    }
    qDebug() <<  endl;
}

bool CarteMifare::estPremierBloc(int numeroBloc) const
{
  if(numeroBloc >= 0 && numeroBloc < NB_BLOCS)
    return ((numeroBloc) % 4 == 0);
  return false;
}

bool CarteMifare::estBlocSecurite(int numeroBloc) const
{
  if(numeroBloc >= 0 && numeroBloc < NB_BLOCS)
      if(numeroBloc == getNumeroBlocSecurite(numeroBloc))
    return true;
  return false;
}

int CarteMifare::getNumeroBlocSecurite(int numeroBloc) const
{
  int trailer_block = -1;
  if(numeroBloc >= 0 && numeroBloc < NB_BLOCS)
  {
    trailer_block = numeroBloc + (3 - (numeroBloc % 4));
  }
  return trailer_block;
}

void CarteMifare::printHexa(const uint8_t *pbtData, const size_t szBytes, QString label /*= ""*/)
{
    size_t  szPos;
    QStringList hex;

    for (szPos = 0; szPos < szBytes; szPos++)
    {
        //hex << QString::number(pbtData[szPos], 16);
        hex <<  QString("%1").arg(pbtData[szPos], 2, 16, QLatin1Char('0'));
    }
    qDebug() << label << "" << hex;
}

QString CarteMifare::toQStringHexa(const uint8_t *pbtData, const size_t szBytes, QString label /*= ""*/, QString delimiteur /*= ""*/)
{
    QString s = label;
    size_t  szPos;
    for (szPos = 0; szPos < szBytes; szPos++)
    {
        s += QString("%1").arg(pbtData[szPos], 2, 16, QLatin1Char('0'));
        if(szPos != szBytes-1)
            s += delimiteur;
    }
    return s;
}

QString CarteMifare::toQStringNum(const uint8_t *pbtData, const size_t szBytes, QString label /*= ""*/)
{
    uint32_t num = 0;
    uint8_t *src = (uint8_t *)pbtData;
    size_t lon = szBytes;
    while (lon--)
    {
      num = (num << 8) | (*src);
      src++;
    }
    QString s = label;
    s += QString::number(num);

    //CarteMifare::printHexa(pbtData, szBytes, "Test : ");
    //qDebug() << "num =" << num;

    return s;
}

void CarteMifare::toBytes(uint32_t data, uint8_t *pbtData, size_t *szBytes)
{
    uint32_t len = sizeof(data);
    uint8_t dest[len];
    int i = 0;
    while (len--)
    {
      dest[len] = (uint8_t)data;
      data >>= 8;
      i++;
    }
    *szBytes = i;
    memcpy(pbtData, dest, i);

    CarteMifare::printHexa(dest, sizeof(data), "Test : ");
    QString s2 = CarteMifare::toQStringHexa(pbtData, i, "0x");
    qDebug() << "Test = " << s2;
}

int CarteMifare::toInt(const uint8_t *pbtData, const size_t szBytes)
{
    uint32_t num = 0;
    uint8_t *src = (uint8_t *)pbtData;
    size_t lon = szBytes;
    while (lon--)
    {
      num = (num << 8) | (*src);
      src++;
    }

    return num;
}

void CarteMifare::decoderACL(const uint8_t acl[]) const
{
    // un secteur = 4 blocs
    const uint8_t       DATA_BLOCK_0 = 0;
    const uint8_t       DATA_BLOCK_1 = 1;
    const uint8_t       DATA_BLOCK_2 = 2;
    const uint8_t       SECTOR_TRAILER = 3;
    uint8_t             permissions[4];
    bool                tableauxBits[3][8]; // la figure 10 cf. page 12 MF1S50YYX.pdf
    uint8_t             _acl[3]; // les 3 octets (temp)
    static const char   droitsSecurite[][64] =   // cf. page 13 MF1S50YYX.pdf
    {
        "Never\tKey A\tKey A\tNever\tKey A\tKey A   | TRAILER :0",
        "Never\tKey A\tKey A\tKey A\tKey A\tKey A   | TRL(TRANSPORT):1",
        "Never\tNever\tKey A\tNever\tKey A\tNever   | TRAILER :2",
        "Never\tKey B\tKyA|B\tKey B\tNever\tKey B   | TRAILER :3",
        "Never\tKey B\tKyA|B\tNever\tNever\tKey B   | TRAILER :4",
        "Never\tNever\tKyA|B\tKey B\tNever\tNever   | TRAILER :5",
        "Never\tNever\tKyA|B\tNever\tNever\tNever   | TRAILER :6",
        "Never\tKey B\tKyA|B\tNever\tNever\tKey B   | TRAILER :7"
    };
    static const char droitsDonnees[][64] =     // cf. page 14 MF1S50YYX.pdf
    {
        "KyA|B\tKA|B\tKA|B\tKA|B\tTransport Config|   :0",
        "KyA|B\tNever\tNever\tKyA|B\tValue Block     | :1",
        "KyA|B\tNever\tNever\tNever\tRead/Write Blk  | :2",
        "Key B\tKey B\tNever\tNever\tRead/Write Blk  | :3",
        "KyA|B\tKey B\tNever\tNever\tRead/Write Blk  | :4",
        "Key B\tNever\tNever\tNever\tRead/Write Blk  | :5",
        "KyA|B\tKey B\tKey B\tKyA|B\tValue Blk       | :6",
        "Never\tNever\tNever\tNever\tRead/Write Blk  | :7",
    };

    memcpy(_acl, acl, 3);

    //QString sb0 = "";
    for (int i = 0; i < 8; i++) // Byte 6
    {
        int j = 0;

        int bit = ( 0x1 & _acl[j] );
        tableauxBits[j][i] = ( bit == 0 ); // complément
        /*if(tableauxBits[j][i] == 1)
            sb0 += "1 ";
        else
            sb0 += "0 ";*/
        _acl[j] = ( _acl[j] >> 1 ); // bit suivant
    }
    //qDebug() << sb0;

    for (int j = 1; j < 3; j++) // Byte 7 et 8
    {
        //QString sb = "";
        for (int i = 0; i < 8; i++)
        {
            int bit = ( 0x1 & _acl[j] );
            if(j == 1 && i < 4)
                tableauxBits[j][i] = ( bit == 0 ); // complément (les 4 bits de poids faible de byte 7)
            else
                tableauxBits[j][i] = ( bit != 0 ); // pas de complément
            /*if(tableauxBits[j][i] == 1)
                sb += "1 ";
            else
                sb += "0 ";*/
            _acl[j] = ( _acl[j] >> 1 ); // bit suivant
        }
        //qDebug() << sb;
    }

    // les trois bits C1, C2 et C3 pour chaque bloc
    permissions[DATA_BLOCK_0] = bin2byte(tableauxBits[1][4], tableauxBits[0][4], tableauxBits[2][4]);
    permissions[DATA_BLOCK_1] = bin2byte(tableauxBits[1][5], tableauxBits[0][5], tableauxBits[2][5]);
    permissions[DATA_BLOCK_2] = bin2byte(tableauxBits[1][6], tableauxBits[0][6], tableauxBits[2][6]);
    permissions[SECTOR_TRAILER] = bin2byte(tableauxBits[1][7], tableauxBits[0][7], tableauxBits[2][7]);

    // affichage du décodage
    qDebug() << "";
    qDebug() << "       C1 C2 C3\tread\twrite\tincr.\td/x/r\tApp";
    qDebug() << "------------------------------------------------------------------------------";

    qDebug() << "BLK 0: " + bin2str(tableauxBits[1][4], tableauxBits[0][4], tableauxBits[2][4]) + "\t" + droitsDonnees[permissions[DATA_BLOCK_0]];
    qDebug() << "BLK 1: " + bin2str(tableauxBits[1][5], tableauxBits[0][5], tableauxBits[2][5]) + "\t" + droitsDonnees[permissions[DATA_BLOCK_1]];
    qDebug() << "BLK 2: " + bin2str(tableauxBits[1][6], tableauxBits[0][6], tableauxBits[2][6]) + "\t"+ droitsDonnees[permissions[DATA_BLOCK_2]];
    qDebug() << "";
    qDebug() << "                KEY A\t\tAccess Bits\tKEY B";
    qDebug() << "       C1 C2 C3\tread\twrite\tread\twrite\tread\twrite";
    qDebug() << "------------------------------------------------------------------------------";
    qDebug() << "BLK 3: " + bin2str(tableauxBits[1][7], tableauxBits[0][7], tableauxBits[2][7]) + "\t"+ droitsSecurite[permissions[SECTOR_TRAILER]];

    /*QString s = " 8 7 6 5 4 3 2 1";
    qDebug() << s;
    for (int i = 0; i < 3; i++)
    {
        QString s = "";
        //for (int j = 0; j < 8; j++)
        for (int j = 7; j >=0 ; j--)
        {
            s += " " + QString::number((tableauxBits[i][j] == true ? 1 : 0));
        }
        qDebug() << s + " : 0x" + CarteMifare::toQStringHexa((uint8_t *)&acl[i], 1);
    }*/
}

QString CarteMifare::bin2str(bool C1, bool C2, bool C3) const
{
    QString s = "";
    if(C1 == true)
        s += "1  ";
    else
        s += "0  ";
    if(C2 == true)
        s += "1  ";
    else
        s += "0  ";
    if(C3 == true)
        s += "1";
    else
        s += "0";

    return s;
}

uint8_t CarteMifare::bin2byte(bool C1, bool C2, bool C3) const
{
    uint8_t dd = (C1 == true ? 1 : 0);
    dd = (dd << 1);
    dd = (dd | (C2 == true ? 1 : 0));
    dd = (dd << 1);
    dd = (dd | ( C3 == true ? 1 : 0));

    return dd;
}

/*void CarteMifare::setUid(uint32_t uid)
{
    this->uid = uid;
}

void CarteMifare::setUid(QString uid)
{
    this->uid = uid.toInt();
}

void CarteMifare::setUidFromHexa(QString uid)
{
    bool ok = 0;
    this->uid = uid.toInt(&ok, 16);
}*/

uint32_t CarteMifare::getUid()
{
    if(nbBlocsLus == 0)
        return 0;
    return CarteMifare::toQStringNum(parametres.mpAuthentication.authUid, 4).toUInt();
}

QString CarteMifare::getUidToString()
{
    if(nbBlocsLus == 0)
        return QString("");
    return CarteMifare::toQStringNum(parametres.mpAuthentication.authUid, 4);
}

QString CarteMifare::getUidToHexa()
{
    if(nbBlocsLus == 0)
        return QString("");
    return CarteMifare::toQStringHexa(parametres.mpAuthentication.authUid, 4);
}
