#include "LecteurNFC.h"

#include <QDebug>

LecteurNFC::LecteurNFC() : carteMifare(NULL), lecteur(NULL), contexte(NULL), init(false), present(false), ouvert(false), nomLecteur(""), nomPeripherique("")
{
    carteMifare = new CarteMifare; // utilise les clés par défaut
    initialiser();
}

LecteurNFC::LecteurNFC(CarteMifare *carteMifare) : carteMifare(carteMifare), lecteur(NULL), contexte(NULL), init(false), present(false), ouvert(false), nomLecteur(""), nomPeripherique("")
{
    initialiser();
}

LecteurNFC::~LecteurNFC() 
{
    detacherCarte();
    if(init == true)
        // ferme le périphérique
        nfc_close(lecteur);

    // libère le contexte
    nfc_exit(contexte);
}

void LecteurNFC::reset(uint8_t uid[], bool format /*= false*/)
{
    MifareBloc  pDataIn, pDataOut;
    uint8_t     _uid[12];

    configurer();

    envoyerCommandeISO(MC_REQA, &pDataOut);
    qDebug() << CarteMifare::toQStringHexa(pDataOut.mbData.data, 2, "ATQA : ", " ");

    // Anti-collision
    pDataIn.mbData.data[0] = 0x20;
    envoyerCommandeISOCarte(MC_SELECT_ALL, &pDataIn, &pDataOut);
    qDebug() << CarteMifare::toQStringHexa(pDataOut.mbFabricant.uid, 4, "UID : ", " ") << "(" << CarteMifare::toQStringNum(pDataOut.mbFabricant.uid, 4) << ")";
    qDebug() << CarteMifare::toQStringHexa(&pDataOut.mbFabricant.bcc, 1, "BCC : 0x") << "(" << CarteMifare::toQStringNum(&pDataOut.mbFabricant.bcc, 1) << ")";
    uint8_t calcul_bcc = pDataOut.mbData.data[0] ^ pDataOut.mbData.data[1] ^ pDataOut.mbData.data[2] ^ pDataOut.mbData.data[3];
    qDebug() << CarteMifare::toQStringHexa(&calcul_bcc, 1, "Verification BCC : 0x") << "(" << CarteMifare::toQStringNum(&calcul_bcc, 1) << ")";
    if(calcul_bcc != pDataOut.mbFabricant.bcc)
        qDebug() << "Attention : echec de la verification du BCC !";

    // sauvegarde l'uid
    memcpy(_uid, pDataOut.mbFabricant.uid, 4);

    // CL1 Select-Command
    uint8_t selectTag[9] = { 0x93, 0x70, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
    memcpy(selectTag + 2, pDataOut.mbData.data, 5);
    iso14443a_crc_append(selectTag, 7);
    memcpy(pDataIn.mbData.data, selectTag + 1, 8);
    envoyerCommandeISOCarte(MC_CL1_SELECT, &pDataIn, &pDataOut);
    qDebug() << CarteMifare::toQStringHexa(&pDataOut.mbData.data[0], 1, "SAK : 0x");

    uint8_t sak = pDataOut.mbData.data[0];
    // CL2 ?
    if (sak & 0x04) // Cascade bit ?
    {
        qDebug() << "Attention : CL2 ou plus !";
        //szCL = 2;//or more
        if (uid[0] != 0x88)
        {
            qDebug() << "Attention : le cascade bit = 1 mais CT != 0x88 !";
        }
        return;
    }
    // SAK_FLAG_ATS_SUPPORTED ?
    if (pDataOut.mbData.data[0] & 0x20)
    {
        qDebug() << "ISO 14443A-4 supporte !";
    }
    // CL2, CL3 ... TODO

    // ISO14443A Anti-Collision
    uint8_t  halt[4] = { 0x50, 0x00, 0x00, 0x00 };
    iso14443a_crc_append(halt, 2);
    memcpy(pDataIn.mbData.data, halt + 1, 3);
    envoyerCommandeISOCarte(MC_HALT, &pDataIn, &pDataOut);
    //if(!envoyerCommandeISOCarte(MC_HALT, &pDataIn, &pDataOut))
    //    return;

    envoyerCommandeISO(MC_UNLOCK1, &pDataOut);

    if (format) // Delete all data (set to 0xFF) and reset ACLs to default.
    {
        envoyerCommandeISOCarte(MC_WIPE, &pDataIn, &pDataOut);
        envoyerCommandeISOCarte(MC_HALT, &pDataIn, &pDataOut);
        envoyerCommandeISO(MC_UNLOCK1, &pDataOut);
    }

    envoyerCommandeISOCarte(MC_UNLOCK2, &pDataIn, &pDataOut);

    uint8_t specialWrite[4] = { 0xa0,  0x00,  0x5f,  0xb1 };
    memcpy(pDataIn.mbData.data, specialWrite + 1, 3);
    envoyerCommandeISOCarte(MC_SPECIAL_WRITE, &pDataIn, &pDataOut);

    // reset UID
    memcpy(pDataIn.mbData.data, uid, 4);
    envoyerCommandeISOCarte(MC_DATA, &pDataIn, &pDataOut);

    if (format) // Delete all data (set to 0xFF) and reset ACLs to default.
    {
        for (int i = 3 ; i < 64 ; i += 4)
        {
            specialWrite[1] = (char) i;
            iso14443a_crc_append(specialWrite, 2);
            memcpy(pDataIn.mbData.data, specialWrite + 1, 3);
            envoyerCommandeISOCarte(MC_SPECIAL_WRITE, &pDataIn, &pDataOut);

            envoyerCommandeISOCarte(MC_BLANK, &pDataIn, &pDataOut);
        }
    }
}

void LecteurNFC::setCarteMifare(CarteMifare *carteMifare)
{
    this->carteMifare = carteMifare;
}

CarteMifare *LecteurNFC::getCarteMifare()
{
    return carteMifare;
}

void LecteurNFC::detacherCarte()
{
    if(this->carteMifare != NULL)
        delete carteMifare;
}

void LecteurNFC::afficherDetails() const
{
    if(init == true)
    {
        qDebug() << "Lecteur : " << getNomLecteur();
        qDebug() << "Peripherique : " << getNomPeripherique();
        qDebug() << "Details : ";
        char *s;
        nfc_device_get_information_about(lecteur, &s);
        qDebug() << s;
        nfc_free(s);
    }
}

QString LecteurNFC::getNomLecteur() const
{
    if(init == true)
        return nomLecteur;
    return "";
}

QString LecteurNFC::getLibVersion() const
{
    if(init == true)
        return QString(nfc_version());
    return "";
}

QString LecteurNFC::getNomPeripherique() const
{
    if(init == true)
        return nomPeripherique;
    return "";
}

QString LecteurNFC::getDetails() const
{
    if(init == true)
    {
        char *s;
        nfc_device_get_information_about(lecteur, &s);
        QString details(s);
        nfc_free(s);
        return details;
    }
    return "";
}

QString LecteurNFC::getInfosCarte(bool verbose/*=false*/) const
{
    nfc_modulation nm;
    nfc_target ant[1];
    int res = 0;
    QString infos = "";

    if(carteMifare == NULL || init == false)
        return infos;

    nm = carteMifare->getNfcModulation();

    if ((res = nfc_initiator_list_passive_targets(lecteur, nm, ant, 1)) >= 0)
    {
        if (res > 0)
        {
            infos += "Carte ISO14443A detectee ";
            char *s;
            str_nfc_target(&s, &ant[0], verbose);
            infos += QString(s);
            nfc_free(s);

            if (nfc_initiator_select_passive_target(lecteur, nm, NULL, 0, &ant[0]) > 0)
            {
                infos += CarteMifare::toQStringHexa(ant[0].nti.nai.abtAtqa, 2, "ATQA (SENS_RES): ", " ");
                //CarteMifare::printHexa(ant[0].nti.nai.abtAtqa, 2, "ATQA (SENS_RES): ");

                QString uid;
                if(ant[0].nti.nai.abtUid[0] == 0x08)
                    uid = "UID (NFCID3): ";
                else
                    uid = "UID (NFCID1): ";
                infos += CarteMifare::toQStringHexa(ant[0].nti.nai.abtUid, ant[0].nti.nai.szUidLen, uid, " ");
                //CarteMifare::printHexa(ant[0].nti.nai.abtUid, ant[0].nti.nai.szUidLen, uid);

                infos += CarteMifare::toQStringHexa(&ant[0].nti.nai.btSak, 1, "SAK (SEL_RES): ", " ");
                //CarteMifare::printHexa(&ant[0].nti.nai.btSak, 1, "SAK (SEL_RES): ");
                if (ant[0].nti.nai.szAtsLen)
                {
                    infos += CarteMifare::toQStringHexa(ant[0].nti.nai.abtAts, ant[0].nti.nai.szAtsLen, "ATS (ATR): ", " ");
                    //CarteMifare::printHexa(ant[0].nti.nai.abtAts, ant[0].nti.nai.szAtsLen, "ATS (ATR): ");
                }
            }
            else
            {
                nfc_perror(lecteur, "nfc_initiator_select_passive_target");
            }
        }
        else
        {
            qDebug() << "Aucune carte ISO14443A detectee ";
            infos += "Aucune carte ISO14443A detectee ";
        }
    }
    return infos;
}

void LecteurNFC::afficherInfosCarte(bool verbose/*=false*/) const
{
    nfc_modulation nm;
    nfc_target ant[1];
    int res = 0;

    if(carteMifare == NULL || init == false)
        return;

    nm = carteMifare->getNfcModulation();

    if ((res = nfc_initiator_list_passive_targets(lecteur, nm, ant, 1)) >= 0)
    {
        if (res > 0)
        {
            qDebug() << "Carte ISO14443A detectee ";
            char *s;
            str_nfc_target(&s, &ant[0], verbose);
            //printf("%s", s);
            qDebug() <<  s;
            nfc_free(s);

            if (nfc_initiator_select_passive_target(lecteur, nm, NULL, 0, &ant[0]) > 0)
            {
                qDebug() << CarteMifare::toQStringHexa(ant[0].nti.nai.abtAtqa, 2, "ATQA (SENS_RES): ", " ");
                //CarteMifare::printHexa(ant[0].nti.nai.abtAtqa, 2, "ATQA (SENS_RES): ");

                QString uid;
                if(ant[0].nti.nai.abtUid[0] == 0x08)
                    uid = "UID (NFCID3): ";
                else
                    uid = "UID (NFCID1): ";
                qDebug() << CarteMifare::toQStringHexa(ant[0].nti.nai.abtUid, ant[0].nti.nai.szUidLen, uid, " ");
                //CarteMifare::printHexa(ant[0].nti.nai.abtUid, ant[0].nti.nai.szUidLen, uid);

                qDebug() << CarteMifare::toQStringHexa(&ant[0].nti.nai.btSak, 1, "SAK (SEL_RES): ", " ");
                //CarteMifare::printHexa(&ant[0].nti.nai.btSak, 1, "SAK (SEL_RES): ");
                if (ant[0].nti.nai.szAtsLen)
                {
                    qDebug() << CarteMifare::toQStringHexa(ant[0].nti.nai.abtAts, ant[0].nti.nai.szAtsLen, "ATS (ATR): ", " ");
                    //CarteMifare::printHexa(ant[0].nti.nai.abtAts, ant[0].nti.nai.szAtsLen, "ATS (ATR): ");
                }
            }
            else
            {
                nfc_perror(lecteur, "nfc_initiator_select_passive_target");
            }
        }
        else
        {
            qDebug() << "Aucune carte ISO14443A detectee ";
        }
        qDebug() << endl;
    }    
}

bool LecteurNFC::lireCarteMifare()
{
    MifareParametres pAuthentication, pData;
    bool auth = false;
    bool res = false;

    if(carteMifare == NULL || init == false)
        return false;

    if(!estCartePresente())
        return false;

    for(int numeroBloc = 0; numeroBloc < carteMifare->getNbBlocs(); numeroBloc++)
    {
        MifareBloc bloc;

        // on doit s'authentifier pour chaque secteur (4 blocs) lus
        if(carteMifare->estPremierBloc(numeroBloc))
            auth = authentifierCarteMifare(numeroBloc);

        if (auth == true)
        {
            pAuthentication = carteMifare->getParametres();
            // on lit le bloc (16 octets)
            res = envoyerCommandeCarteMifare(MC_READ, numeroBloc, &pAuthentication, &pData);
            if (res == true)
            {
                memcpy(bloc.mbData.data, pData.mpData.data, 16);
                carteMifare->setBloc(numeroBloc, bloc);
                //carteMifare->afficherBloc(numeroBloc);
            }
        }
        else
        {
            continue;
            //return false;
        }
        emit progress((((double)numeroBloc+1.)/((double)carteMifare->getNbBlocs()))*100.);
    }
    //carteMifare->afficherBlocs();

    return true;
}

bool LecteurNFC::lireBlocCarteMifare(int numeroBloc)
{
    MifareBloc bloc;
    MifareParametres pAuthentication, pData;
    bool auth = false;
    bool res = false;

    if(carteMifare == NULL || init == false)
        return false;

    if(!estCartePresente())
        return false;

    // on doit s'authentifier pour lire ce bloc
    auth = authentifierCarteMifare(numeroBloc);

    if (auth == true)
    {
        pAuthentication = carteMifare->getParametres();
        // on lit le bloc (16 octets)
        res = envoyerCommandeCarteMifare(MC_READ, numeroBloc, &pAuthentication, &pData);
        if (res == true)
        {
            memcpy(bloc.mbData.data, pData.mpData.data, 16);
            carteMifare->setBloc(numeroBloc, bloc);
            //carteMifare->afficherBloc(numeroBloc);
        }
    }
    else
    {
        return false;
    }

    return true;
}

bool LecteurNFC::lireBlocCarteMifare(int numeroBloc, uint8_t cle[], char typeAccesCle/*='A'*/)
{
    MifareBloc bloc;
    MifareParametres pAuthentication, pData;
    bool auth = false;
    bool res = false;

    if(carteMifare == NULL || init == false)
        return false;

    if(!estCartePresente())
        return false;

    // on doit s'authentifier pour lire ce bloc
    auth = authentifierCarteMifare(numeroBloc, cle, typeAccesCle);

    if (auth == true)
    {
        pAuthentication = carteMifare->getParametres();
        // on lit le bloc (16 octets)
        res = envoyerCommandeCarteMifare(MC_READ, numeroBloc, &pAuthentication, &pData);
        if (res == true)
        {
            memcpy(bloc.mbData.data, pData.mpData.data, 16);
            carteMifare->setBloc(numeroBloc, bloc);
            //carteMifare->afficherBloc(numeroBloc);
        }
    }
    else
    {
        return false;
    }

    return true;
}

bool LecteurNFC::ecrireBlocCarteMifare(int numeroBloc, const MifareBloc &bloc)
{
    MifareParametres pData;
    bool auth = false;
    bool res = false;

    if(carteMifare == NULL || init == false)
        return false;

    if(!estCartePresente())
        return false;

    // on doit s'authentifier pour écrire ce bloc
    auth = authentifierCarteMifare(numeroBloc);

    if (auth == true)
    {
        // on écrit le bloc (16 octets)
        memcpy(pData.mpData.data, bloc.mbData.data, 16);
        res = envoyerCommandeCarteMifare(MC_WRITE, numeroBloc, &pData, &pData);
        if (res == true)
        {
            carteMifare->setBloc(numeroBloc, bloc);
            //carteMifare->afficherBloc(numeroBloc);
        }
    }
    else
    {
        return false;
    }

    return true;
}

bool LecteurNFC::ecrireBlocCarteMifare(int numeroBloc, const MifareBloc &bloc, uint8_t cle[], char typeAccesCle/*='A'*/)
{
    MifareParametres pData;
    bool auth = false;
    bool res = false;

    if(carteMifare == NULL || init == false)
        return false;

    if(!estCartePresente())
        return false;

    // on doit s'authentifier pour écrire ce bloc
    auth = authentifierCarteMifare(numeroBloc, cle, typeAccesCle);

    if (auth == true)
    {
        // on écrit le bloc (16 octets)
        memcpy(pData.mpData.data, bloc.mbData.data, 16);
        res = envoyerCommandeCarteMifare(MC_WRITE, numeroBloc, &pData, &pData);
        if (res == true)
        {
            carteMifare->setBloc(numeroBloc, bloc);
            //carteMifare->afficherBloc(numeroBloc);
        }
    }
    else
    {
        return false;
    }

    return true;
}

bool LecteurNFC::ecrireCleCarteMifare(int numeroBloc, uint8_t nouvelleCle[], char typeAccesCle/*='A'*/)
{
    MifareParametres pData;
    bool auth = false;
    bool res = false;

    if(carteMifare == NULL || init == false)
        return false;

    if(!estCartePresente())
        return false;

    if(!carteMifare->estBlocSecurite(numeroBloc))
        return false;

    // on récupère les datas de ce bloc
    this->lireBlocCarteMifare(numeroBloc);
    MifareBloc bloc = carteMifare->getBloc(numeroBloc);

    MifareCle cleAcces;
    if(typeAccesCle == 'A')
    {
        cleAcces = carteMifare->getCle('A', numeroBloc);
    }
    else if(typeAccesCle == 'B')
    {
        cleAcces = carteMifare->getCle('B', numeroBloc);
    }
    else
        return false;

    // on doit s'authentifier avant d'écrire
    auth = authentifierCarteMifare(numeroBloc, &cleAcces.key[0], typeAccesCle);

    if (auth == true)
    {
        memcpy(pData.mpData.data, bloc.mbData.data, 16);

        if(typeAccesCle == 'A')
        {
            memcpy(pData.mpData.data, nouvelleCle, 6);
            MifareCle cle = carteMifare->getCle('B', numeroBloc);
            memcpy(&pData.mpData.data[10], cle.key, 6);
        }
        else if(typeAccesCle == 'B')
        {
            memcpy(&pData.mpData.data[10], nouvelleCle, 6);
            MifareCle cle = carteMifare->getCle('A', numeroBloc);
            memcpy(pData.mpData.data, cle.key, 6);
        }
        else
            return false;

        //qDebug() << "Ecriture sur le secteur " + QString("%1").arg((numeroBloc/4), 2, 10, QLatin1Char('0')) + " " + CarteMifare::toQStringHexa(pData.mpData.data, 16, ", bloc " + QString("%1").arg(numeroBloc, 2, 10, QLatin1Char('0')) + " : ", " ");

        res = envoyerCommandeCarteMifare(MC_WRITE, numeroBloc, &pData, &pData);
        if (res == true)
        {
            carteMifare->setCle(typeAccesCle, nouvelleCle, numeroBloc);
            carteMifare->setBloc(numeroBloc, bloc);
            //carteMifare->afficherBloc(numeroBloc);
        }
    }
    else
    {
        return false;
    }

    return true;
}

bool LecteurNFC::ecrireDroitsAccesCarteMifare(int numeroBloc, uint8_t droitsAcces[], char typeAccesCle/*='A'*/)
{
    MifareParametres pData;
    bool auth = false;
    bool res = false;

    if(carteMifare == NULL || init == false)
        return false;

    if(!estCartePresente())
        return false;

    if(!carteMifare->estBlocSecurite(numeroBloc))
        return false;

    // on récupère les datas de ce bloc
    this->lireBlocCarteMifare(numeroBloc);
    MifareBloc bloc = carteMifare->getBloc(numeroBloc);

    MifareCle cle;
    if(typeAccesCle == 'A')
    {
        cle = carteMifare->getCle('A', numeroBloc);
    }
    else if(typeAccesCle == 'B')
    {
        cle = carteMifare->getCle('B', numeroBloc);
    }
    else
        return false;

    // on doit s'authentifier avant d'écrire
    auth = authentifierCarteMifare(numeroBloc, &cle.key[0], typeAccesCle);

    if (auth == true)
    {
        memcpy(pData.mpData.data, bloc.mbData.data, 16);

        MifareCle cleA = carteMifare->getCle('A', numeroBloc);
        memcpy(pData.mpData.data, cleA.key, 6);
        memcpy(&pData.mpData.data[6], droitsAcces, 4);
        MifareCle cleB = carteMifare->getCle('B', numeroBloc);
        memcpy(&pData.mpData.data[10], cleB.key, 6);

        //qDebug() << "Ecriture sur le secteur " + QString("%1").arg((numeroBloc/4), 2, 10, QLatin1Char('0')) + " " + CarteMifare::toQStringHexa(pData.mpData.data, 16, ", bloc " + QString("%1").arg(numeroBloc, 2, 10, QLatin1Char('0')) + " : ", " ");
        //carteMifare->decoderACL(droitsAcces);

        res = envoyerCommandeCarteMifare(MC_WRITE, numeroBloc, &pData, &pData);
        if (res == true)
        {
            carteMifare->setBloc(numeroBloc, bloc);
            //carteMifare->afficherBloc(numeroBloc);
        }
    }
    else
    {
        return false;
    }

    return true;
}

bool LecteurNFC::estCarteMifare()
{
    nfc_modulation nm;
    nfc_target ant[1];

    if(carteMifare == NULL || init == false)
        return false;

    nm = carteMifare->getNfcModulation();

    if (nfc_initiator_select_passive_target(lecteur, nm, NULL, 0, &ant[0]) < 1)
    {
        qDebug() << "Carte non detectee";
        return false;
    }
    if ((ant[0].nti.nai.btSak & 0x08) == 0)
    {
        qDebug() << "Ce n'est pas une carte Mifare Classic !";
        return false;
    }
    return true;
}

bool LecteurNFC::estCartePresente()
{
    nfc_modulation nm;
    nfc_target ant[1];

    if(carteMifare == NULL || init == false)
        return false;

    nm = carteMifare->getNfcModulation();

    if ((nfc_initiator_list_passive_targets(lecteur, nm, ant, 1)) > 0)
    {
        if (nfc_initiator_select_passive_target(lecteur, nm, NULL, 0, &ant[0]) < 1)
        {
            qDebug() << "Carte non detectee";
            return false;
        }
    }
    else
    {
        qDebug() << "Carte non detectee";
        return false;
    }
    return true;
}

bool LecteurNFC::estPresent()
{
    return present;
}

bool LecteurNFC::estOuvert()
{
    return ouvert;
}

bool LecteurNFC::estInitialise()
{
    return init;
}

bool LecteurNFC::authentifierCarteMifare(int numeroBloc)
{
    MifareParametres pAuthentication, pData;
    nfc_modulation nm;
    nfc_target ant[1];
    char typeAccesCle;
    bool res = false;

    if(carteMifare == NULL || init == false)
        return false;

    nm = carteMifare->getNfcModulation();

    if(!estCartePresente())
        return false;

    if (nfc_initiator_select_passive_target(lecteur, nm, NULL, 0, &ant[0]) > 0)
    {
        pAuthentication = carteMifare->getParametres();
        typeAccesCle = carteMifare->getTypeAccesCle();
        MifareCle cle = carteMifare->getCle(typeAccesCle, numeroBloc);
        memcpy(pAuthentication.mpAuthentication.key, cle.key, sizeof(cle.key));
        memcpy(pAuthentication.mpAuthentication.authUid, ant[0].nti.nai.abtUid + ant[0].nti.nai.szUidLen - 4, sizeof(pAuthentication.mpAuthentication.authUid));
        carteMifare->setParametres(pAuthentication);
        //carteMifare->afficherParametresAuthentication();
    }
    else return false;

    // Authentification
    if(typeAccesCle == 'A')
        res = envoyerCommandeCarteMifare(MC_AUTH_A, numeroBloc, &pAuthentication, &pData);
    else if(typeAccesCle == 'B')
        res = envoyerCommandeCarteMifare(MC_AUTH_B, numeroBloc, &pAuthentication, &pData);
    else
        return false;

    return res;
}

bool LecteurNFC::authentifierCarteMifare(int numeroBloc, uint8_t cle[], char typeAccesCle/*='A'*/)
{
    MifareParametres pAuthentication, pData;
    nfc_modulation nm;
    nfc_target ant[1];
    char _typeAccesCle;
    bool res;

    if(carteMifare == NULL || init == false)
        return false;

    nm = carteMifare->getNfcModulation();

    if(!estCartePresente())
        return false;

    if (nfc_initiator_select_passive_target(lecteur, nm, NULL, 0, &ant[0]) > 0)
    {
        pAuthentication = carteMifare->getParametres();
        // on modifie la cle pour ce numero de bloc et on l'utilise pour s'authentifier
        carteMifare->setCle(typeAccesCle, cle, numeroBloc);
        _typeAccesCle = carteMifare->getTypeAccesCle();
        MifareCle _cle = carteMifare->getCle(typeAccesCle, numeroBloc);
        memcpy(pAuthentication.mpAuthentication.key, _cle.key, sizeof(_cle.key));
        memcpy(pAuthentication.mpAuthentication.authUid, ant[0].nti.nai.abtUid + ant[0].nti.nai.szUidLen - 4, sizeof(pAuthentication.mpAuthentication.authUid));
        carteMifare->setParametres(pAuthentication);

        carteMifare->afficherParametresAuthentication();
    }
    else return false;

    // Authentification
    if(_typeAccesCle == 'A')
        res = envoyerCommandeCarteMifare(MC_AUTH_A, numeroBloc, &pAuthentication, &pData);
    else if(_typeAccesCle == 'B')
        res = envoyerCommandeCarteMifare(MC_AUTH_B, numeroBloc, &pAuthentication, &pData);
    else
        return false;

    return res;
}

bool LecteurNFC::envoyerCommandeCarteMifare(const MifareCommande commande, const uint8_t bloc, const MifareParametres *pDataIn, MifareParametres *pDataOut)
{
    uint8_t  rx[265];
    size_t   longueur;
    uint8_t  cmd[265];

    if(init == false)
        return false;

    cmd[0] = commande;
    cmd[1] = bloc;     // The block address (1K=0x00..0x39, 4K=0x00..0xff)

    switch (commande)
    {
            // ces commndes n'ont pas de paramétres
    case    MC_READ:
    case    MC_STORE:
    case    MC_SELECT_ALL:
      longueur = 0;
      break;
            // commande authentication (API-ACR122U-2.02.pdf : page 13)
    case    MC_AUTH_A:
    case    MC_AUTH_B:
      longueur = sizeof(MifareParametresAuth);
      break;
            // commande écriture
    case    MC_WRITE:
      longueur = sizeof(MifareParametresData);
      break;
            // Value command
    case    MC_DECREMENT:
    case    MC_INCREMENT:
    case    MC_TRANSFER:
      longueur = sizeof(MifareParametresValue);
      break;
            // commande inconnue !
    default:
      return false;
      break;
    }

    // copie les paramétres dans le message de commande
    if (longueur)
        memcpy(cmd + 2, (uint8_t *)pDataIn, longueur);

    // désactive les commandes brutes (raw)
    int res;
    if ((res = nfc_device_set_property_bool(lecteur, NP_EASY_FRAMING, true)) < 0)
    {
        nfc_perror(lecteur, "nfc_device_set_property_bool");
        return false;
    }
    // transmission du message de commande
    qDebug() << CarteMifare::toQStringHexa(cmd, 2 + longueur, "   -> Send : ", " ");
    if ((res = nfc_initiator_transceive_bytes(lecteur, cmd, 2 + longueur, rx, sizeof(rx), -1))  < 0)
    {
        nfc_perror(lecteur, "nfc_initiator_transceive_bytes");
        return false;
    }

    // une lecture ? alors on copie les données reçues
    if (commande == MC_READ)
    {
        if (res == 16)
        {
            qDebug() << CarteMifare::toQStringHexa(rx, 16, "   -> Receive : ", " ");
            memcpy(pDataOut->mpData.data, rx, 16);
        }
        else
        {
            return false;
        }
    }

    return true;
}

bool LecteurNFC::envoyerCommandeISOCarte(const MifareCommande commande, const MifareBloc *pDataIn, MifareBloc *pDataOut)
{
    uint8_t  rx[265];
    size_t   longueur;
    uint8_t  cmd[265];
    QString  label;
    int      res;
    bool     special = false;
    uint8_t  data[18] = { 0x01,  0x23,  0x45,  0x67,  0x00,  0x08,  0x04,  0x00,  0x46,  0x59,  0x25,  0x58,  0x49,  0x10,  0x23,  0x02,  0x23,  0xeb };
    uint8_t  blank[18] = { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x07, 0x80, 0x69, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x36, 0xCC };

    if(init == false)
        return false;

    //cmd[0] = commande;

    switch (commande)
    {
    case    MC_SPECIAL_WRITE:
        label = "   -> Send (WRITE) : ";
        cmd[0] = MC_WRITE;
        longueur = 3;
        break;
    case    MC_SELECT_ALL:
        label = "   -> Send (SELECT ALL) : ";
        cmd[0] = MC_SELECT_ALL;
        longueur = 1;
        break;
    case    MC_CL1_SELECT:
        label = "   -> Send (CL1 SELECT) : ";
        cmd[0] = 0x93;
        longueur = 8;
        break;
    case MC_HALT:
        label = "   -> Send (HALT) : ";
        cmd[0] = MC_HALT;
        longueur = 3;
        break;
    case MC_WIPE:
        label = "   -> Send (WIPE) : ";
        cmd[0] = MC_WIPE;
        longueur = 0;
        break;
    case MC_UNLOCK2:
        label = "   -> Send (UNLOCK2) : ";
        cmd[0] = MC_UNLOCK2;
        longueur = 0;
        break;
    case MC_DATA:
        label = "   -> Send (DATA) : ";
        longueur = 0;
        memcpy(data, pDataIn->mbData.data, 4);
        iso14443a_crc_append(data, 16);
        memcpy(cmd, data, 18);
        special = true;
        break;
    case MC_BLANK:
        label = "   -> Send (BLANK) : ";
        longueur = 0;
        memcpy(cmd, blank, 18);
        special = true;
        break;
    default: // commande inconnue !
      return false;
      break;
    }

    // copie les paramétres dans le message de commande
    if (longueur)
        memcpy(cmd + 1, (uint8_t *)pDataIn, longueur);

    // transmission du message de commande
    if(special == false)
    {
        qDebug() << CarteMifare::toQStringHexa(cmd, 1 + longueur, label, " ");
        if ((res = nfc_initiator_transceive_bytes(lecteur, cmd, 1 + longueur, rx, sizeof(rx), -1))  < 0)
        {
            //qDebug() << "res = " << res;
            switch(res)
            {
            case NFC_ERFTRANS : // -20
                qDebug() << "Erreur : autorisation refusee !";
                break;
            }
            nfc_perror(lecteur, "nfc_initiator_transceive_bytes");
            return false;
        }
    }
    else
    {
        qDebug() << CarteMifare::toQStringHexa(cmd, 18, label, " ");
        if ((res = nfc_initiator_transceive_bytes(lecteur, cmd, 18, rx, sizeof(rx), -1))  < 0)
        {
            //qDebug() << "res = " << res;
            switch(res)
            {
            case NFC_ERFTRANS : // -20
                qDebug() << "Erreur : autorisation refusee !";
                break;
            }
            nfc_perror(lecteur, "nfc_initiator_transceive_bytes");
            return false;
        }
    }

    if (res)
    {
        qDebug() << CarteMifare::toQStringHexa(rx, res, "   -> Receive : ", " ");
        memcpy(pDataOut->mbData.data, rx, res);
    }

    return true;
}

bool LecteurNFC::envoyerCommandeISO(const MifareCommande commande, MifareBloc *pDataOut)
{
    uint8_t  rx[265];
    size_t   longueur, longueurRx;
    uint8_t  cmd[1];
    int      res;
    QString  label;

    if(init == false)
        return false;

    cmd[0] = commande;

    switch (commande)
    {
    case    MC_REQA:    // 7 bits specifiés dans ISO 14443A
        label = "   -> Send (REQA) : ";
        longueur = 7; // en bits
    case    MC_UNLOCK1:
        label = "   -> Send (UNLOCK1) : ";
        longueur = 7; // en bits
        break;
            // commande inconnue !
    default:
        return false;
        break;
    }

    // transmission de la trame de bits (sans parité)
    qDebug() << CarteMifare::toQStringHexa(cmd, 1, label, " ");
    if ((res = nfc_initiator_transceive_bits(lecteur, cmd, longueur, NULL, rx, sizeof(rx), NULL)) < 0)
        return false;

    longueurRx = (size_t)res;
    memcpy(pDataOut->mbData.data, rx, longueurRx);
    qDebug() << CarteMifare::toQStringHexa(rx, longueurRx, "   -> Receive : ", " ");

    return true;
}

void LecteurNFC::initialiser()
{
    init = false;

    // initialise la libnfc et le contexte
    nfc_init(&contexte);
    if (contexte == NULL)
    {
        qDebug() << "Erreur : impossible d'initialiser la libnfc (malloc) !";
        return;
    }

    // recherche un périphérique lecteur
    size_t szDeviceFound = nfc_list_devices(contexte, peripherique, 1);
    if (szDeviceFound == 0)
    {
        present = false;
        qDebug() << "Erreur : aucun lecteur present !";
        return;
    }
    present = true;

    // ouvre le premier périphérique disponible
    //lecteur = nfc_open(contexte, NULL);
    // ouvre le premier périphérique disponible de la liste
    lecteur = nfc_open(contexte, peripherique[0]);
    if (lecteur == NULL)
    {
        ouvert = false;
        qDebug() << "Erreur : impossible d'ouvrir le peripherique !";
        nfc_perror(lecteur, "nfc_open");
        return;
    }
    ouvert = true;

    // dans une communication, il y a un initiateur (initiator) et une cible (target)
    // fixe le périphérique ouvert dans le mode "initiator"
    if (nfc_initiator_init(lecteur) < 0)
    {
        nfc_perror(lecteur, "nfc_initiator_init");
        return;
    }

    nomLecteur = QString(nfc_device_get_name(lecteur));
    nomPeripherique = QString(peripherique[0]);

    init = true;
}

void LecteurNFC::configurer()
{
    if(init == false)
        return;

    if (nfc_initiator_init(lecteur) < 0)
    {
        nfc_perror(lecteur, "nfc_initiator_init");
        return;
    }
    /*
    // Drop the field for a while, so can be reset
    if (nfc_device_set_property_bool(lecteur, NP_ACTIVATE_FIELD, true) < 0)
    {
        nfc_perror(lecteur, "nfc_device_set_property_bool activate field");
        return;
    }
    */
    /*
    // Let the reader only try once to find a tag
    if (nfc_device_set_property_bool(lecteur, NP_INFINITE_SELECT, false) < 0)
    {
        nfc_perror(lecteur, "nfc_device_set_property_bool infinite select");
        return;
    }
    */

    // fixe le contrôle du CRC et de parité
    if (nfc_device_set_property_bool(lecteur, NP_HANDLE_CRC, false) < 0)
    {
        nfc_perror(lecteur, "nfc_device_set_property_bool crc");
        return;
    }

    /*
    if (nfc_device_set_property_bool(lecteur, NP_HANDLE_PARITY, true) < 0)
    {
        nfc_perror(lecteur, "nfc_device_set_property_bool parity");
        return;
    }*/


    // désactive les commandes brutes (raw)
    if (nfc_device_set_property_bool(lecteur, NP_EASY_FRAMING, false) < 0)
    {
        nfc_perror(lecteur, "nfc_device_set_property_bool");
        return;
    }


    // désactive l'autoswitching 14443-4
    if (nfc_device_set_property_bool(lecteur, NP_AUTO_ISO14443_4, false) < 0)
    {
        nfc_perror(lecteur, "nfc_device_set_property_bool");
        return;
    }

    /*
    // Enable the field so more power consuming cards can power themselves up
    if (nfc_device_set_property_bool(lecteur, NP_ACTIVATE_FIELD, true) < 0)
    {
        nfc_perror(lecteur, "nfc_device_set_property_bool activate field");
        return;
    }*/
}

void LecteurNFC::selectionner()
{
    nfc_modulation nm;
    nfc_target ant[1];

    if(carteMifare == NULL || init == false)
        return;

    nm = carteMifare->getNfcModulation();

    nfc_initiator_select_passive_target(lecteur, nm, NULL, 0, &ant[0]);
}
