#ifndef LECTEURNFC_H
#define LECTEURNFC_H

#include <nfc/nfc.h>

#include <QStringList>
#include <QString>

#include <stdint.h>

#include "CarteMifare.h"

class LecteurNFC : public QObject
{
    Q_OBJECT
public:
    explicit LecteurNFC();
    explicit LecteurNFC(CarteMifare *carteMifare);
    ~LecteurNFC();
    
    void    reset(uint8_t uid[], bool format=false); // reset UID et formate la carte
    void    setCarteMifare(CarteMifare *carteMifare);
    CarteMifare *getCarteMifare();
    void    detacherCarte();
    void    afficherDetails() const;
    QString getNomLecteur() const;
    QString getLibVersion() const;
    QString getNomPeripherique() const;
    QString getDetails() const;
    QString getInfosCarte(bool verbose=false) const;
    void    afficherInfosCarte(bool verbose=false) const;
    bool    lireCarteMifare();
    bool    lireBlocCarteMifare(int numeroBloc);
    bool    lireBlocCarteMifare(int numeroBloc, uint8_t cle[], char typeAccesCle='A');
    bool    ecrireBlocCarteMifare(int numeroBloc, const MifareBloc &bloc);
    bool    ecrireBlocCarteMifare(int numeroBloc, const MifareBloc &bloc, uint8_t cle[], char typeAccesCle='A');
    bool    ecrireCleCarteMifare(int numeroBloc, uint8_t nouvelleCle[], char typeAccesCle='A');
    bool    ecrireDroitsAccesCarteMifare(int numeroBloc, uint8_t droitsAcces[], char typeAccesCle='A');
    bool    estCarteMifare();
    bool    estCartePresente();
    bool    estPresent();
    bool    estOuvert();
    bool    estInitialise();
    
private:
    CarteMifare *carteMifare;
    nfc_device *lecteur;
    nfc_context *contexte;
    nfc_connstring peripherique[1]; // un seul suffit !
    bool init;
    bool present;
    bool ouvert;
    QString nomLecteur;
    QString nomPeripherique;

    bool authentifierCarteMifare(int numeroBloc);
    bool authentifierCarteMifare(int numeroBloc, uint8_t cle[], char typeAccesCle='A');
    bool envoyerCommandeCarteMifare(const MifareCommande commande, const uint8_t bloc, const MifareParametres *pDataIn, MifareParametres *pDataOut);
    bool envoyerCommandeISOCarte(const MifareCommande commande, const MifareBloc *pDataIn, MifareBloc *pDataOut);
    bool envoyerCommandeISO(const MifareCommande commande, MifareBloc *pDataOut);
    void initialiser();
    void configurer();
    void selectionner();

signals:
    void progress(int);
};

#endif // LECTEURNFC_H
