#include "ihm.h"
#include <QDebug>

IHM::IHM(QWidget *parent) :
    QWidget(parent)
{
    lecteurNFC = new LecteurNFC;

    messages = new QTextEdit(this);
    messages->setReadOnly(true);
    numBloc = new QSpinBox(this);
    numBloc->setMinimum(0);
    numBloc->setMaximum(63);
    numBloc->setValue(4); // premier bloc du secteur 1
    numBloc->setEnabled(false);
    QLabel *labelBloc = new QLabel(QString::fromUtf8("Numéro de bloc :"), this);
    boutonLire = new QPushButton(QString::fromUtf8("Lire"), this);
    boutonLireCarte = new QPushButton(QString::fromUtf8("Dump carte"), this);
    boutonEcrire = new QPushButton(QString::fromUtf8("Écrire"), this);    
    boutonEffacer = new QPushButton(QString::fromUtf8("Effacer"), this);
    boutonLire->setEnabled(false);
    boutonLireCarte->setEnabled(false);
    boutonEcrire->setEnabled(false);
    boutonEffacer->setEnabled(false);
    leDonnees = new QLineEdit(this);
    leDonnees->setEnabled(false);
    leDonnees->setFixedWidth(leDonnees->sizeHint().width());
    progressBar1 = new QProgressBar(this);
    progressBar1->setValue(0);

    QVBoxLayout *layoutPrincipal = new QVBoxLayout;
    QHBoxLayout *layoutMessages = new QHBoxLayout;
    QHBoxLayout *layoutBoutons = new QHBoxLayout;
    layoutMessages->addWidget(messages);
    layoutBoutons->addWidget(progressBar1);
    layoutBoutons->addWidget(boutonLireCarte);
    layoutBoutons->addStretch();
    layoutBoutons->addWidget(labelBloc);
    layoutBoutons->addWidget(numBloc);
    layoutBoutons->addWidget(boutonLire);    
    layoutBoutons->addWidget(leDonnees);
    layoutBoutons->addWidget(boutonEcrire);
    layoutBoutons->addWidget(boutonEffacer);
    layoutPrincipal->addLayout(layoutMessages);
    layoutPrincipal->addLayout(layoutBoutons);
    setLayout(layoutPrincipal);

    setWindowTitle(QString::fromUtf8("Test Lecteur NFC 0.9"));
    setFixedWidth(1080);
    setFixedHeight(640);

    if(lecteurNFC->estInitialise())
    {
        messages->append(QString::fromUtf8("<b>Version libnfc : %1</b>").arg(lecteurNFC->getLibVersion()));
        messages->append(QString::fromUtf8("<b>Lecteur : %1</b>").arg(lecteurNFC->getNomLecteur()));
        messages->append(QString::fromUtf8("Peripherique : %1").arg(lecteurNFC->getNomPeripherique()));
        messages->append(QString::fromUtf8("Détails : %1").arg(lecteurNFC->getDetails()));
    }
    else
    {
        if(!lecteurNFC->estPresent())
            messages->append(QString::fromUtf8("<font color=red>Lecteur non détecté ! Veuillez brancher un lecteur ACR122U et relancer l'application.</font>"));
        else if(!lecteurNFC->estOuvert())
            messages->append(QString::fromUtf8("<font color=red>Lecteur non ouvert ! Veuillez vérifier les droits d'accès et relancer l'application.</font>"));
    }

    cartePresente = false;
    timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(lireCarte()));
    timer->start(500);

    connect(this, SIGNAL(progress(int)), progressBar1, SLOT(setValue(int)));
    connect(lecteurNFC, SIGNAL(progress(int)), progressBar1, SLOT(setValue(int)));
    connect(boutonLire, SIGNAL(clicked()), this, SLOT(lire()));
    connect(boutonLireCarte, SIGNAL(clicked()), this, SLOT(lireTout()));
    connect(boutonEcrire, SIGNAL(clicked()), this, SLOT(ecrire()));
    connect(boutonEffacer, SIGNAL(clicked()), this, SLOT(effacer()));
}

IHM::~IHM()
{
    if(timer->isActive())
    {
        timer->stop();
    }

    delete lecteurNFC;
}

void IHM::lireCarte()
{
    if(!lecteurNFC->estCartePresente())
    {
        if(cartePresente)
            messages->append(QString::fromUtf8("\n<b>Carte retirée !<b>\n"));
        cartePresente = false;
        boutonLire->setEnabled(false);
        boutonLireCarte->setEnabled(false);
        boutonEcrire->setEnabled(false);
        boutonEffacer->setEnabled(false);
        leDonnees->setEnabled(false);
        numBloc->setEnabled(false);
        return;
    }
    if(cartePresente == false)
    {
        messages->append(QString::fromUtf8("Veuillez patienter ...\n"));
        lecteurNFC->lireCarteMifare();
        messages->append(QString::fromUtf8("%1\n").arg(lecteurNFC->getInfosCarte(false)));
        messages->append(QString::fromUtf8("<font color=green>UID : %1 (%2)</font>").arg(lecteurNFC->getCarteMifare()->getUid()).arg(lecteurNFC->getCarteMifare()->getUidToHexa()));
        cartePresente = true;
        boutonLire->setEnabled(true);
        boutonLireCarte->setEnabled(true);
        boutonEcrire->setEnabled(true);
        boutonEffacer->setEnabled(true);
        leDonnees->setEnabled(true);
        numBloc->setEnabled(true);
    }
}

void IHM::lire()
{
    if(!lecteurNFC->estCartePresente())
    {
        return;
    }

    // affiche le contenu du bloc x
    messages->append(QString::fromUtf8("<font color=blue>%1 %2</font>").arg(lecteurNFC->getCarteMifare()->afficherBloc(numBloc->value())).arg(lecteurNFC->getCarteMifare()->getBlocToString(numBloc->value())));
}

void IHM::lireTout()
{
    if(!lecteurNFC->estCartePresente())
    {
        return;
    }

    progressBar1->setValue(0);
    // on efface ce qui a été lu précedemment
    lecteurNFC->getCarteMifare()->effacerBlocsLus();
    for(int i = 0; i < lecteurNFC->getCarteMifare()->getNbBlocs(); i++)
    {
        // lit le bloc i de la carte présente sur le lecteur
        lecteurNFC->lireBlocCarteMifare(i);
        emit progress((((double)i+1.)/((double)lecteurNFC->getCarteMifare()->getNbBlocs()))*100.);
    }
    for(int i = 0; i < lecteurNFC->getCarteMifare()->getNbBlocs(); i++)
    {
        // affiche le contenu du bloc i
        messages->append(QString::fromUtf8("<font color=blue>%1 %2</font>").arg(lecteurNFC->getCarteMifare()->afficherBloc(i)).arg(lecteurNFC->getCarteMifare()->getBlocToString(i)));
    }
}

void IHM::ecrire()
{
    if(!lecteurNFC->estCartePresente())
    {
        return;
    }

    if(leDonnees->text().size() == 0)
    {
        return;
    }

    int numeroBloc = numBloc->value(); // le numéro de bloc où l'on va écrire
    if(numeroBloc != 0 && !lecteurNFC->getCarteMifare()->estBlocSecurite(numeroBloc))
    {
        // Le bloc
        MifareBloc bloc = lecteurNFC->getCarteMifare()->getBloc(numeroBloc);
        // les données du bloc à écrire sur la carte (on limite à 16)
        QString donneesBloc = leDonnees->text().mid(0, 16);
        // on recopie des données dans le bloc
        memcpy(bloc.mbData.data, qPrintable(donneesBloc), donneesBloc.size());
        // écrit le bloc n sur la carte présente sur le lecteur
        if(lecteurNFC->ecrireBlocCarteMifare(numeroBloc, bloc))
        {
            // affiche après le contenu du bloc n
            lire();
        }
    }
    else
    {
        messages->append(QString::fromUtf8("<font color=red>Bloc non autorisé !</font>"));
    }
}

void IHM::effacer()
{
    if(!lecteurNFC->estCartePresente())
    {
        return;
    }

    int numeroBloc = numBloc->value(); // le numéro de bloc où l'on va écrire
    if(numeroBloc != 0 && !lecteurNFC->getCarteMifare()->estBlocSecurite(numeroBloc))
    {
        MifareBloc bloc; // le bloc à écrire sur la carte
        static uint8_t donneesBlocVierge[]  = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};

        // on recopie des données vierges dans le bloc
        memcpy(bloc.mbData.data, donneesBlocVierge, 16); // que des 0
        // écrit le bloc n sur la carte présente sur le lecteur
        if(lecteurNFC->ecrireBlocCarteMifare(numeroBloc, bloc))
        {
            // affiche après le contenu du bloc n
            lire();
        }
    }
    else
    {
        messages->append(QString::fromUtf8("<font color=red>Bloc non autorisé !</font>"));
    }
}


/*IHM::test()
{
    // affiche le contenu du bloc 0 de la carte
    carteMifare->afficherBlocFabricant();
    // affiche le contenu du bloc sécurité associé au bloc x
    carteMifare->afficherBlocSecurite(x);


    int c = 7; // un numéro de bloc où l'on va écrire la nouvelle clé

    // sauf la première fois !
    carteMifare->setCle('B', cle, c);
    // écrit la nouvelle clé B dans le boc c
    if(lecteurNFC.ecrireCleCarteMifare(c, cle, 'B'))
    {
        // affiche le contenu du bloc sécurité c
        carteMifare->afficherBlocSecurite(c);
    }

    // écrit les nouveaux droits d'acccès dans le boc c
    qDebug() << "Ecriture des nouveaux droits d'acces dans le boc "  << c;
    if(lecteurNFC.ecrireDroitsAccesCarteMifare(c, droitsAcces, 'B'))
    {
        // affiche le contenu du bloc sécurité c
        carteMifare->afficherBlocSecurite(c);
    }

}
*/
